@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2013-2017 Canonical, Ltd.
+ * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,57 +16,64 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
 #define MAX_FSTAT_THREADS	(4)
 #define FSTAT_LOOPS		(16)
 
-static const char *opt_fstat_dir = "/dev";
 static volatile bool keep_running;
 static sigset_t set;
 
+static const stress_help_t help[] = {
+	{ NULL,	"fstat N",	  "start N workers exercising fstat on files" },
+	{ NULL,	"fstat-dir path", "fstat files in the specified directory" },
+	{ NULL,	"fstat-ops N",	  "stop after N fstat bogo operations" },
+	{ NULL,	NULL,		  NULL }
+};
+
 /* paths we should never stat */
-static const char *blacklist[] = {
+static const char *blocklist[] = {
 	"/dev/watchdog"
 };
 
+#define IGNORE_STAT	0x0001
+#define IGNORE_LSTAT	0x0002
+#define IGNORE_STATX	0x0004
+#define IGNORE_FSTAT	0x0008
+#define IGNORE_ALL	0x000f
+
 /* stat path information */
 typedef struct stat_info {
-	char		*path;		/* path to stat */
-	bool		ignore;		/* true to ignore this path */
-	bool		access;		/* false if we can't access path */
 	struct stat_info *next;		/* next stat_info in list */
-} stat_info_t;
+	char		*path;		/* path to stat */
+	uint16_t	ignore;		/* true to ignore this path */
+	bool		access;		/* false if we can't access path */
+} stress_stat_info_t;
 
 /* Thread context information */
 typedef struct ctxt {
-	const args_t 	*args;		/* Stressor args */
-	stat_info_t 	*si;		/* path stat information */
-	const uid_t	euid;		/* euid of process */
-} ctxt_t;
+	const stress_args_t *args;	/* Stressor args */
+	stress_stat_info_t *si;		/* path stat information */
+	const uid_t euid;		/* euid of process */
+	const int bad_fd;		/* bad/invalid fd */
+} stress_ctxt_t;
 
-void stress_set_fstat_dir(const char *opt)
+static int stress_set_fstat_dir(const char *opt)
 {
-	opt_fstat_dir = opt;
+	return stress_set_setting("fstat-dir", TYPE_ID_STR, opt);
 }
 
 /*
  *  handle_fstat_sigalrm()
  *      catch SIGALRM
  */
-static void MLOCKED handle_fstat_sigalrm(int dummy)
+static void MLOCKED_TEXT handle_fstat_sigalrm(int signum)
 {
-	(void)dummy;
+	(void)signum;
 
 	keep_running = false;
-	g_keep_stressing_flag = false;
+	keep_stressing_set_flag(false);
 }
 
 /*
@@ -76,26 +84,50 @@ static bool do_not_stat(const char *filename)
 {
 	size_t i;
 
-	for (i = 0; i < SIZEOF_ARRAY(blacklist); i++) {
-		if (!strncmp(filename, blacklist[i], strlen(blacklist[i])))
+	for (i = 0; i < SIZEOF_ARRAY(blocklist); i++) {
+		if (!strncmp(filename, blocklist[i], strlen(blocklist[i])))
 			return true;
 	}
 	return false;
 }
 
-static void stress_fstat_helper(const ctxt_t *ctxt)
+static void stress_fstat_helper(const stress_ctxt_t *ctxt)
 {
 	struct stat buf;
-	stat_info_t *si = ctxt->si;
+#if defined(AT_EMPTY_PATH) &&	\
+    defined(AT_SYMLINK_NOFOLLOW)
+	shim_statx_t bufx;
+#endif
+	stress_stat_info_t *si = ctxt->si;
 
 	if ((stat(si->path, &buf) < 0) && (errno != ENOMEM)) {
-		si->ignore = true;
-		return;
+		si->ignore |= IGNORE_STAT;
 	}
 	if ((lstat(si->path, &buf) < 0) && (errno != ENOMEM)) {
-		si->ignore = true;
-		return;
+		si->ignore |= IGNORE_LSTAT;
 	}
+#if defined(AT_EMPTY_PATH) &&	\
+    defined(AT_SYMLINK_NOFOLLOW)
+	/* Heavy weight statx */
+	if ((shim_statx(AT_EMPTY_PATH, si->path, AT_SYMLINK_NOFOLLOW,
+		SHIM_STATX_ALL, &bufx) < 0) && (errno != ENOMEM)) {
+		si->ignore |= IGNORE_STATX;
+	}
+
+	/* invalid dfd in statx */
+	VOID_RET(int, shim_statx(-1, "baddfd", AT_SYMLINK_NOFOLLOW, SHIM_STATX_ALL, &bufx));
+
+	/* invalid path in statx */
+	VOID_RET(int, shim_statx(AT_EMPTY_PATH, "", AT_SYMLINK_NOFOLLOW, SHIM_STATX_ALL, &bufx));
+
+	/* invalid mask in statx */
+	VOID_RET(int, shim_statx(AT_EMPTY_PATH, si->path, ~0, SHIM_STATX_ALL, &bufx));
+
+	/* invalid flags in statx */
+	VOID_RET(int, shim_statx(AT_EMPTY_PATH, si->path, AT_SYMLINK_NOFOLLOW, ~0U, &bufx));
+#else
+	UNEXPECTED
+#endif
 	/*
 	 *  Opening /dev files such as /dev/urandom
 	 *  may block when running as root, so
@@ -110,9 +142,18 @@ static void stress_fstat_helper(const ctxt_t *ctxt)
 			return;
 		}
 		if ((fstat(fd, &buf) < 0) && (errno != ENOMEM))
-			si->ignore = true;
+			si->ignore |= IGNORE_FSTAT;
 		(void)close(fd);
 	}
+
+	/* Exercise stat on an invalid path, ENOENT */
+	VOID_RET(int, stat("", &buf));
+
+	/* Exercise lstat on an invalid path, ENOENT */
+	VOID_RET(int, lstat("", &buf));
+
+	/* Exercise fstat on an invalid fd, EBADF */
+	VOID_RET(int, fstat(ctxt->bad_fd, &buf));
 }
 
 #if defined(HAVE_LIB_PTHREAD)
@@ -124,33 +165,25 @@ static void stress_fstat_helper(const ctxt_t *ctxt)
 static void *stress_fstat_thread(void *ctxt_ptr)
 {
 	static void *nowt = NULL;
-	uint8_t stack[SIGSTKSZ + STACK_ALIGNMENT];
-	const ctxt_t *ctxt = (const ctxt_t *)ctxt_ptr;
+	const stress_ctxt_t *ctxt = (const stress_ctxt_t *)ctxt_ptr;
 
 	/*
 	 *  Block all signals, let controlling thread
 	 *  handle these
 	 */
-	sigprocmask(SIG_BLOCK, &set, NULL);
+#if !defined(__APPLE__)
+	(void)sigprocmask(SIG_BLOCK, &set, NULL);
+#endif
 
-	/*
-	 *  According to POSIX.1 a thread should have
-	 *  a distinct alternative signal stack.
-	 *  However, we block signals in this thread
-	 *  so this is probably just totally unncessary.
-	 */
-	memset(stack, 0, sizeof(stack));
-	if (stress_sigaltstack(stack, SIGSTKSZ) < 0)
-		return &nowt;
-
-	while (keep_running && g_keep_stressing_flag) {
+	while (keep_running && keep_stressing_flag()) {
 		size_t i;
 
-		for (i = 0; i < FSTAT_LOOPS; i++)  {
-			if (!g_keep_stressing_flag)
+		for (i = 0; i < FSTAT_LOOPS; i++) {
+			if (!keep_stressing_flag())
 				break;
 			stress_fstat_helper(ctxt);
 		}
+		(void)shim_sched_yield();
 	}
 
 	return &nowt;
@@ -161,23 +194,24 @@ static void *stress_fstat_thread(void *ctxt_ptr)
  *  stress_fstat_threads()
  *	create a bunch of threads to thrash a file
  */
-static void stress_fstat_threads(const args_t *args, stat_info_t *si, const uid_t euid)
+static void stress_fstat_threads(const stress_args_t *args, stress_stat_info_t *si, const uid_t euid)
 {
 	size_t i;
 #if defined(HAVE_LIB_PTHREAD)
 	pthread_t pthreads[MAX_FSTAT_THREADS];
 	int ret[MAX_FSTAT_THREADS];
 #endif
-	ctxt_t ctxt = {
+	stress_ctxt_t ctxt = {
 		.args 	= args,
 		.si 	= si,
-		.euid	= euid
+		.euid	= euid,
+		.bad_fd = stress_get_bad_fd()
 	};
 
 	keep_running = true;
 #if defined(HAVE_LIB_PTHREAD)
-	memset(ret, 0, sizeof(ret));
-	memset(pthreads, 0, sizeof(pthreads));
+	(void)memset(ret, 0, sizeof(ret));
+	(void)memset(pthreads, 0, sizeof(pthreads));
 
 	for (i = 0; i < MAX_FSTAT_THREADS; i++) {
 		ret[i] = pthread_create(&pthreads[i], NULL,
@@ -185,7 +219,7 @@ static void stress_fstat_threads(const args_t *args, stat_info_t *si, const uid_
 	}
 #endif
 	for (i = 0; i < FSTAT_LOOPS; i++) {
-		if (!g_keep_stressing_flag)
+		if (!keep_stressing_flag())
 			break;
 		stress_fstat_helper(&ctxt);
 	}
@@ -194,7 +228,7 @@ static void stress_fstat_threads(const args_t *args, stat_info_t *si, const uid_
 #if defined(HAVE_LIB_PTHREAD)
 	for (i = 0; i < MAX_FSTAT_THREADS; i++) {
 		if (ret[i] == 0)
-			pthread_join(pthreads[i], NULL);
+			(void)pthread_join(pthreads[i], NULL);
 	}
 #endif
 }
@@ -203,22 +237,25 @@ static void stress_fstat_threads(const args_t *args, stat_info_t *si, const uid_
  *  stress_fstat()
  *	stress system with fstat
  */
-int stress_fstat(const args_t *args)
+static int stress_fstat(const stress_args_t *args)
 {
-	stat_info_t *si;
-	static stat_info_t *stat_info;
+	stress_stat_info_t *si;
+	static stress_stat_info_t *stat_info;
 	struct dirent *d;
 	NOCLOBBER int ret = EXIT_FAILURE;
 	bool stat_some;
 	const uid_t euid = geteuid();
 	DIR *dp;
+	char *fstat_dir = "/dev";
+
+	(void)stress_get_setting("fstat-dir", &fstat_dir);
 
 	if (stress_sighandler(args->name, SIGALRM, handle_fstat_sigalrm, NULL) < 0)
 		return EXIT_FAILURE;
 
-	if ((dp = opendir(opt_fstat_dir)) == NULL) {
+	if ((dp = opendir(fstat_dir)) == NULL) {
 		pr_err("%s: opendir on %s failed: errno=%d: (%s)\n",
-			args->name, opt_fstat_dir, errno, strerror(errno));
+			args->name, fstat_dir, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
@@ -226,13 +263,13 @@ int stress_fstat(const args_t *args)
 	while ((d = readdir(dp)) != NULL) {
 		char path[PATH_MAX];
 
-		if (!g_keep_stressing_flag) {
+		if (!keep_stressing_flag()) {
 			ret = EXIT_SUCCESS;
 			(void)closedir(dp);
 			goto free_cache;
 		}
 
-		(void)snprintf(path, sizeof(path), "%s/%s", opt_fstat_dir, d->d_name);
+		(void)stress_mk_filename(path, sizeof(path), fstat_dir, d->d_name);
 		if (do_not_stat(path))
 			continue;
 		if ((si = calloc(1, sizeof(*si))) == NULL) {
@@ -246,34 +283,38 @@ int stress_fstat(const args_t *args)
 			(void)closedir(dp);
 			goto free_cache;
 		}
-		si->ignore = false;
+		si->ignore = 0;
 		si->access = true;
 		si->next = stat_info;
 		stat_info = si;
 	}
 	(void)closedir(dp);
 
-	sigfillset(&set);
+	(void)sigfillset(&set);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	do {
 		stat_some = false;
 
-		for (si = stat_info; g_keep_stressing_flag && si; si = si->next) {
-			if (!keep_stressing())
+		for (si = stat_info; keep_stressing_flag() && si; si = si->next) {
+			if (!keep_stressing(args))
 				break;
-			if (si->ignore)
+			if (si->ignore == IGNORE_ALL)
 				continue;
 			stress_fstat_threads(args, si, euid);
 
 			stat_some = true;
 			inc_counter(args);
 		}
-	} while (stat_some && keep_stressing());
+	} while (stat_some && keep_stressing(args));
 
 	ret = EXIT_SUCCESS;
 free_cache:
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
 	/* Free cache */
 	for (si = stat_info; si; ) {
-		stat_info_t *next = si->next;
+		stress_stat_info_t *next = si->next;
 
 		free(si->path);
 		free(si);
@@ -282,3 +323,15 @@ free_cache:
 
 	return ret;
 }
+
+static const stress_opt_set_func_t opt_set_funcs[] = {
+	{ OPT_fstat_dir,	stress_set_fstat_dir },
+	{ 0,			NULL }
+};
+
+stressor_info_t stress_fstat_info = {
+	.stressor = stress_fstat,
+	.class = CLASS_FILESYSTEM | CLASS_OS,
+	.opt_set_funcs = opt_set_funcs,
+	.help = help
+};
