@@ -31,8 +31,8 @@ static uint64_t lease_sigio;
 
 static const stress_help_t help[] = {
 	{ NULL,	"lease N",	    "start N workers holding and breaking a lease" },
-	{ NULL,	"lease-ops N",	    "stop after N lease bogo operations" },
 	{ NULL,	"lease-breakers N", "number of lease breaking workers to start" },
+	{ NULL,	"lease-ops N",	    "stop after N lease bogo operations" },
 	{ NULL, NULL,		    NULL }
 };
 
@@ -269,9 +269,10 @@ stressor_info_t stress_lease_info = {
 };
 #else
 stressor_info_t stress_lease_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without fcntl() F_SETLEASE, F_WRLCK or F_UNLCK commands"
 };
 #endif
