@@ -28,10 +28,7 @@ static const stress_help_t help[] = {
 
 static int stress_set_mincore_rand(const char *opt)
 {
-	bool mincore_rand = true;
-
-	(void)opt;
-	return stress_set_setting("mincore-rand", TYPE_ID_BOOL, &mincore_rand);
+	return stress_set_setting_true("mincore-rand", opt);
 }
 
 static const stress_opt_set_func_t opt_set_funcs[] = {
@@ -72,6 +69,32 @@ static int stress_mincore_file(const stress_args_t *args)
 	return fd;
 }
 
+/*
+ *  stress_mincore_expect()
+ *	check for expected return code/errors
+ */
+static void stress_mincore_expect(
+	const stress_args_t *args,
+	const int ret,		/* return value */
+	const int ret_expected,	/* exepected return value */
+	const int err,		/* returned errno */
+	const int err_expected,	/* expected errno */
+	char *msg)		/* test message */
+{
+	if (ret == ret_expected) {
+		if (ret_expected == 0)
+			return;	/* Success! */
+		pr_fail("%s: unexpected success exercising %s\n",
+			args->name, msg);
+	}
+	/* Silently ignore ENOSYS for now */
+	if (err == ENOSYS)
+		return;
+	if (err != err_expected)
+		pr_fail("%s: expected errno %d, got %d instead while exercising %s\n",
+			args->name, err_expected, err, msg);
+}
+
 /*
  *  stress_mincore()
  *	stress mincore system call
@@ -115,7 +138,7 @@ static int stress_mincore(const stress_args_t *args)
 
 		for (i = 0; (i < 100) && keep_stressing_flag(); i++) {
 			int ret, redo = 0;
-			unsigned char vec[1];
+			static unsigned char vec[1];
 
 redo: 			errno = 0;
 			ret = shim_mincore((void *)addr, page_size, vec);
@@ -164,7 +187,7 @@ redo: 			errno = 0;
 #endif
 				ret = shim_mincore((void *)fdmapped, page_size, vec);
 				if (ret < 0) {
-					/* Should no return ENOMEM on a mapped page */
+					/* Should not return ENOMEM on a mapped page */
 					if (errno == ENOMEM) {
 						pr_fail("%s: mincore on address %p failed, errno=$%d (%s)\n",
 							args->name, (void *)fdmapped, errno,
@@ -192,30 +215,58 @@ redo: 			errno = 0;
 				addr += page_size;
 			}
 
-			/*
-			 *  Exercise with zero length, ignore return
-			 */
-			(void)shim_mincore((void *)addr, 0, vec);
+			if (mapped != MAP_FAILED) {
+				/*
+				 *  Exercise with zero length, ignore return
+				 */
+				ret = shim_mincore((void *)mapped, 0, vec);
+				stress_mincore_expect(args, ret, 0, errno, EINVAL,
+					"zero length for vector size");
 
-			/*
-			 *  Exercise with NULL vec, ignore return
-			 */
-			(void)shim_mincore((void *)addr, page_size, NULL);
+#if 0
+				/*
+				 *  Exercise with huge length, ignore return
+				 */
+				ret = shim_mincore((void *)mapped, ~0, vec);
+				stress_mincore_expect(args, ret, 0, errno, ENOMEM,
+					"invalid length for vector size");
+#endif
+
+				/*
+				 *  Exercise with masaligned address, ignore return
+				 */
+				ret = shim_mincore((void *)(mapped + 1), 0, vec);
+				stress_mincore_expect(args, ret, 0, errno, EINVAL,
+					"misaligned address");
+
+				/*
+				 *  Exercise with NULL vec, ignore return
+				 */
+				ret = shim_mincore((void *)mapped, page_size, NULL);
+				stress_mincore_expect(args, ret, 0, errno, EFAULT,
+					"NULL vector address");
+
+				/*
+				 *  Exercise with invalid page
+				 */
+				ret = shim_mincore(mapped, page_size, args->mapped->page_none);
+				stress_mincore_expect(args, ret, 0, errno, EFAULT,
+					"invalid vector address");
+			}
 
 			/*
 			 *  Exercise with NULL address
 			 */
-			(void)shim_mincore(NULL, page_size, vec);
+			ret = shim_mincore(NULL, page_size, vec);
+			stress_mincore_expect(args, ret, 0, errno, ENOMEM,
+				"NULL memory address");
 
 			/*
-			 *  Exercise with zero arguments
+			 *  Exercise with NULL/zero arguments
 			 */
-			(void)shim_mincore(NULL, 0, NULL);
-
-			/*
-			 *  Exercise with invalid page
-			 */
-			(void)shim_mincore(NULL, page_size, args->mapped->page_none);
+			ret = shim_mincore(NULL, 0, NULL);
+			stress_mincore_expect(args, ret, 0, errno, EINVAL,
+				"NULL and zero arguments");
 		}
 		inc_counter(args);
 	} while (keep_stressing(args));
@@ -244,9 +295,10 @@ stressor_info_t stress_mincore_info = {
 };
 #else
 stressor_info_t stress_mincore_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_OS | CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without mincore() system call support"
 };
 #endif
