@@ -242,9 +242,10 @@ static int stress_pidfd_supported(const char *name)
 }
 
 stressor_info_t stress_pidfd_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_INTERRUPT | CLASS_OS,
 	.supported = stress_pidfd_supported,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without pidfd_send_signal() system call"
 };
 #endif
