@@ -266,8 +266,9 @@ stressor_info_t stress_unshare_info = {
 };
 #else
 stressor_info_t stress_unshare_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without unshare() system call"
 };
 #endif
